# China-Taiwan model.R
#
# 20110612 : Initial version -- II
# 20110718 : Revised to do quicker specification searches and save the
#            forecasts -- PTB
# 20120309 : Include MSBVAR models in the results.
# 20120314 : Check serial correlation of the PITs before plotting;
#            sharpness diagrams

# Load the MSBVAR and Zoo libraries
library(MSBVAR)
library(zoo)

# Load the data
load("CHNTWN19982011monthly.RData")

# Setup the data in a single time series matrix
Y <- as.ts(cbind(CHNTWN.monthly[[2]]- CHNTWN.monthly[[1]],
                 CHNTWN.monthly[[4]] - CHNTWN.monthly[[3]]))

colnames(Y) <- c("C2T-M", "T2C-M", "C2T-V", "T2C-V")


## # Plot the data
## library(YaleToolkit)
## pdf("straitsdata.pdf", height=8, width=6)
## sparklines(as.data.frame(Y), times=index(Y),
##            buffer=unit(0.5,"lines"),
##            ylab=gsub("Cp", "", gsub("WN", "", gsub("HN", "", colnames(Y)))))
## dev.off()


# Get a sample and forecast subsets for the data
Y.sample <- window(Y, end=c(2005, 12))   # Y less that last 12 periods
Y6 <- window(Y, end=c(2006, 6))          # Y less the last 6 period
Y12 <- window(Y, end=c(2006,12))         # Full sample

###################################################################
# MSBVARs
###################################################################
set.seed(1984)

## MSV <- msvar(Y.sample, p=1, h=2, niterblkopt=80)

## int <- initialize.msbvar(y=Y.sample, p=1, lambda0=0.9, lambda1=0.25,
##              lambda3=1, lambda4=0.15, lambda5=0, mu5=0, mu6=0,
##              qm=12, prior=0, h=2)

## int$Qhat.start <- MSV$Q
## thetahat.start <- array(NA, c(MSV$m, 1 + MSV$m * MSV$p + MSV$m, MSV$h))
## thetahat.start[,1,1] <- MSV$hreg$Bk[5,,1]
## thetahat.start[,1,2] <- MSV$hreg$Bk[5,,2]
## #thetahat.start[,1,3] <- MSV$hreg$Bk[5,,3]
## thetahat.start[,2:5,1] <- t(MSV$hreg$Bk[1:4,1:4,1])
## thetahat.start[,2:5,2] <- t(MSV$hreg$Bk[1:4,1:4,2])
## #thetahat.start[,2:5,3] <- t(MSV$hreg$Bk[1:4,1:4,3])

## thetahat.start[,6:9,1] <- MSV$hreg$Sigmak[,,1]
## thetahat.start[,6:9,2] <- MSV$hreg$Sigmak[,,2]
## #thetahat.start[,6:9,3] <- MSV$hreg$Sigmak[,,3]

## int$thetahat.start <- thetahat.start

## plot(ts(MSV$fp), plot.type="s", col=1:3)

# Now do the Bayesian version

md <- msbvar(Y.sample, p=1, h=2, lambda0=0.9, lambda1=0.4,
             lambda3=1, lambda4=0.15, lambda5=0, mu5=0, mu6=0,
             qm=12, alpha.prior=c(10,7)*diag(2) + matrix(10,2,2),
             max.iter=100, initialize.opt=NULL)

# Examine the posterior mode estimates to see how they are behaved.
plot(ts(md$fp), plot.type="s", col=1:3)
print(md$Q)

# Gibbs sampling with permutation
MSBVARperm <- gibbs.msbvar(md, N1=5000, N2=5000, permute=TRUE)

plotregimeid(MSBVARperm, type="all")

# Gibbs sampling with identification
MSBVARid <- gibbs.msbvar(md, N1=1000, N2=5000, permute=FALSE,
                         Sigma.idx=4)

plotregimeid(MSBVARid, type="Sigma")

pdf(file="MSBVARregimes.pdf", width=6, height=6)
par(mfrow=c(2,1))
plot(Y.sample, plot.type="s", lty=1:4, ylab="Data")
tsp1 <- tsp(Y.sample)
plot(ts(mean.SS(MSBVARid), start=tsp1[1], freq=tsp1[3]),
     plot.type="s", col=1:2, ylab="Regime Probabilities")

dev.off()

summary(MSBVARid$Q.sample)
print(ts(mean.SS(MSBVARid), start=tsp1[1], freq=tsp1[3]))



## # Simple VAR analysis

## # Step 1: Determine lag length
## print(var.lag.specification(Y.sample, lagmax=12))

## # Step 2: Granger causality analysis
## tmp <- granger.test(Y.sample, p=6)

## # Step 3: Fit a model with 6 lags
## VAR6 <- reduced.form.var(Y.sample, p=6)

## # Step 4: Compute impulse responses
## irfCT <- irf(VAR6, nsteps=12)
## plot(irfCT, varnames=colnames(Y))

## ###################################################################
## # Bayesian VAR Prior Specification Search
## ###################################################################

## results <- SZ.prior.evaluation(window(Y.sample, end=c(2009, 3)),
## 			       p=6,
## 			       lambda0=c(0.6, 0.7, 0.8, 0.9, 1),
## 			       lambda1=c(0.1, 0.15, 0.25),
##  			       lambda3=c(1, 1.5, 2),
## 			       lambda4=c(0.1, 0.15, 0.25, 0.5),
## 			       lambda5=0,
## 			       mu5=0,
## 			       mu6=0, z=NULL,
## 			       nu=ncol(Y)+1, qm=12,
## 			       prior=0,
## 			       nstep=12,
## 			       y.future=window(Y, start=c(2009, 3),
##                                end=c(2010, 2)))

## # Now plot the RMSE and marginal posterior of the data for each of the
## # 6 period forecasts as a function of the prior parameters.

## attach(as.data.frame(results))

## pdf(file="ChinaTaiwan-priorevaluationL4RMSE.pdf")
## xyplot(RMSE ~ lambda0 | lambda1 + lambda3, groups=lambda4)
## dev.off()

## pdf(file="ChinaTaiwan-priorevaluationL4logMDD.pdf")
## xyplot(logMDD ~ lambda0 | lambda1 + lambda3, groups=lambda4)
## dev.off()

## pdf(file="ChinaTaiwan-priorevaluationL4LLF.pdf")
## xyplot(LLF ~ lambda0 | lambda1 + lambda3, groups=lambda4)
## dev.off()

###################################################################
# Fit the FINAL BVARs
###################################################################
BVAR <- szbvar(Y.sample, p=6, lambda0=0.9, lambda1=0.25,
               lambda3=1, lambda4=0.15, lambda5=0, mu5=0, mu6=0,
               qm=12, prior=0)

BVAR.flat <- szbvar(Y.sample, p=6, lambda0=0.9, lambda1=0.1,
                        lambda3=1, lambda4=0.15, lambda5=0, mu5=0, mu6=0,
                        qm=12, prior=2)

###################################################################
# Univariate AR models of lag by AIC
###################################################################
univar <- vector(ncol(Y.sample), mode="list")
for(i in 1:ncol(Y.sample))
{
    univar[[i]] <- ar.ols(Y.sample[,i], order.max=12, demean=FALSE,
                          intercept=TRUE)

}



###################################################################
# FORECAST densities
# Generate forecast densities for each model and forecast horizon:
# 1) VAR
# 2) BVAR
# 3) Univariate ARs
# 4) MSBVAR
###################################################################
nsteps <- 6
burnin <- 1000
gibbs <- 5000

set.seed(11023)

###################################################################
# 6 step forecasts
###################################################################
# Forecast BVAR
BVAR.forc6 <- uc.forecast(BVAR, nsteps, burnin, gibbs)

# Forecast VAR
BVAR.flat.forc6 <- uc.forecast(BVAR.flat, nsteps, burnin, gibbs)

# Forecast for the univariate AR
univar.forc6 <- vector(length(univar), mode="list")
for(i in 1:length(univar))
{
    univar.forc6[[i]] <- predict(univar[[i]], n.ahead=nsteps,
                              se.fit=TRUE)
}

tmp <- matrix(unlist(univar.forc6), 6, 8)
univar6.mean <- tmp[,seq(1,ncol(tmp),2)]
univar6.sd <- tmp[,seq(2,ncol(tmp),2)]

# Now simulate univariate AR forecast densities
univar.forc6.ensemble <- array(0, c(gibbs, nsteps, ncol(Y)))

for(i in 1:ncol(Y.sample))
{
    univar.forc6.ensemble[,,i] <- rmvnorm(gibbs, mean=univar6.mean[,i],
                                          sigma=diag(univar6.sd[,i]^2))
}

###################################################################
# code works up to here!
###################################################################





###################################################################
# Forecast the MSBVAR over 6 periods
#source("~/MSBVARlocal/devel-unstable/msbvarforecast.R")
###################################################################
msbvar.forc6 <- forecast.MSBVAR(MSBVARid, k=nsteps, N1=burnin,
                               N2=gibbs)


###################################################################
# 12 step forecasts
###################################################################
nsteps <- 12
# Forecast BVAR
BVAR.forc12 <- uc.forecast(BVAR, nsteps, burnin, gibbs)

# Forecast VAR
BVAR.flat.forc12 <- uc.forecast(BVAR.flat, nsteps, burnin, gibbs)

# Forecast for the univariate AR
univar.forc12 <- vector(length(univar), mode="list")
for(i in 1:length(univar))
{
    univar.forc12[[i]] <- predict(univar[[i]], n.ahead=nsteps,
                              se.fit=TRUE)
}

tmp <- matrix(unlist(univar.forc12), 12, 8)
univar12.mean <- tmp[,seq(1,ncol(tmp),2)]
univar12.sd <- tmp[,seq(2,ncol(tmp),2)]

# Now simulate univariate AR forecast densities
univar.forc12.ensemble <- array(0, c(gibbs, nsteps, ncol(Y)))

for(i in 1:ncol(Y.sample))
{
    univar.forc12.ensemble[,,i] <- rmvnorm(gibbs, mean=univar12.mean[,i],
                                           sigma=diag(univar12.sd[,i]^2))
}

###################################################################
# Forecast the MSBVAR over 12 periods
###################################################################
msbvar.forc12 <- forecast.MSBVAR(MSBVARid, k=nsteps, N1=burnin,
                               N2=gibbs)

###################################################################
# Summarize the forecasts and their CIs
###################################################################
BVAR.forc12.quant <- apply(BVAR.forc12$forecast, c(2,3), quantile,
                           probs=c(0.16, 0.5, 0.84))
BVAR.forc12.mean <- apply(BVAR.forc12$forecast, c(2,3), mean)
BVAR.forc12.sd <- apply(BVAR.forc12$forecast, c(2,3), sd)

BVAR.flat.forc12.quant <- apply(BVAR.flat.forc12$forecast, c(2,3), quantile,
                           probs=c(0.16, 0.5, 0.84))
BVAR.flat.forc12.mean <- apply(BVAR.flat.forc12$forecast, c(2,3), mean)
BVAR.flat.forc12.sd <- apply(BVAR.flat.forc12$forecast, c(2,3), sd)

MSBVAR.forc12.quant <- apply(msbvar.forc12$forecasts, c(2,3), quantile,
                             probs=c(0.16, 0.5, 0.84))

MSBVAR.forc12.mean <- apply(msbvar.forc12$forecasts, c(2,3), mean)
MSBVAR.forc12.sd <- apply(msbvar.forc12$forecasts, c(2,3), sd)


BVAR.forc6.quant <- apply(BVAR.forc6$forecast, c(2,3), quantile,
                           probs=c(0.16, 0.5, 0.84))
BVAR.forc6.mean <- apply(BVAR.forc6$forecast, c(2,3), mean)
BVAR.forc6.sd <- apply(BVAR.forc6$forecast, c(2,3), sd)


BVAR.flat.forc6.quant <- apply(BVAR.flat.forc6$forecast, c(2,3), quantile,
                           probs=c(0.16, 0.5, 0.84))
BVAR.flat.forc6.mean <- apply(BVAR.flat.forc6$forecast, c(2,3), mean)
BVAR.flat.forc6.sd <- apply(BVAR.flat.forc6$forecast, c(2,3), sd)

MSBVAR.forc6.quant <- apply(msbvar.forc6$forecasts, c(2,3), quantile,
                             probs=c(0.16, 0.5, 0.84))

MSBVAR.forc6.mean <- apply(msbvar.forc6$forecasts, c(2,3), mean)
MSBVAR.forc6.sd <- apply(msbvar.forc6$forecasts, c(2,3), sd)



###################################################################
# Plots of the forecasts for each data series vis-a-vis the real data
# and some error bands
###################################################################
Ytmp1 <- window(Y6, start=c(2010,4))
Ytmp2 <- window(Y12, start=c(2010,4))

# Set up a good color palette
pall <- c("cyan", "magenta", "orange", "darkblue")
vnames <- colnames(Y6)

#pdf(file="straitsforecasts.pdf", width=6, height=8)

par(mfcol=c(2,2), mar=c(3,5,1,2))
# 6 period case
for(i in 1:ncol(Y))
{
    # Gather up the forecasts so we can plot them
    mn <- cbind(BVAR.forc6.quant[2,,i], BVAR.flat.forc6.quant[2,,i],
                univar6.mean[,i], MSBVAR.forc6.mean[,i])

    se1 <- cbind(BVAR.forc6.quant[1,,i], BVAR.flat.forc6.quant[1,,i],
                 mn[,3] - univar6.sd[,i], MSBVAR.forc6.quant[1,,i])
    se2 <- cbind(BVAR.forc6.quant[3,,i], BVAR.flat.forc6.quant[3,,i],
                 mn[,3] + univar6.sd[,i], MSBVAR.forc6.quant[3,,i])

    se1 <- hasTsp(se1); tsp(se1) <- tsp(Ytmp1[,i])
    se2 <- hasTsp(se2); tsp(se2) <- tsp(Ytmp1[,i])
    ylim <- range(c(se1, se2, Ytmp1[,i]))

    plot(cbind(Ytmp1[,i], mn, se1, se2), plot.type="single",
         col=c("black", rep(pall, times=4)),
         lwd=2, lty=c(rep(1, 5), rep(2,8)), ylim=ylim, xlab="",
         ylab=vnames[i])
}

# 12 period case
pdf(file="straitsforecasts.pdf", width=6, height=5)

#par(mfcol=c(2,2), mar=c(3,4,2,1), cex=0.8, xpd=TRUE)

layout(matrix(c(1,2,3,4,5,5), 2, 3))
par(mai=c(0.35, 0.35, 0.5, 0.15))
for(i in 1:ncol(Y))
{
    # Gather up the forecasts so we can plot them
    mn <- cbind(BVAR.forc12.quant[2,,i], BVAR.flat.forc12.quant[2,,i],
                univar12.mean[,i], MSBVAR.forc12.mean[,i])

    se1 <- cbind(BVAR.forc12.quant[1,,i], BVAR.flat.forc12.quant[1,,i],
                 mn[,3] - univar12.sd[,i], MSBVAR.forc12.quant[1,,i])
    se2 <- cbind(BVAR.forc12.quant[3,,i], BVAR.flat.forc12.quant[3,,i],
                 mn[,3] + univar12.sd[,i], MSBVAR.forc12.quant[3,,i])
    se1 <- hasTsp(se1); tsp(se1) <- tsp(Ytmp2[,i])
    se2 <- hasTsp(se2); tsp(se2) <- tsp(Ytmp2[,i])
    ylim <- range(c(se1, se2, Ytmp2[,i]))

    plot(cbind(Ytmp2[,i], mn, se1, se2), plot.type="single",
         col=c("black", rep(pall, times=4)),
         lwd=2, lty=c(rep(1,5), rep(2,8)), ylim=ylim, xlab="",
         main=vnames[i], ylab="")
}

# Now draw legend
par(mar=c(0, 0, 0, 0))
plot.new()
legend('center', legend=c("Data", "BVAR informed median forecast",
                 "BVAR informed 68% CI",
                 "BVAR flat median forecast",
                 "BVAR flat 68% CI",
                 "Univariate AR median forecast",
                 "Univariate AR 68% CI",
                 "MS-BVAR median forecast",
                 "MS-BVAR 68% CI"),
       col=c("black", rep(pall, each=2)), lty=c(1,rep(c(1,2), 4)), lwd=2)

dev.off()


###################################################################
# FORECAST COMPARISONS
###################################################################
# Find the RMSE and MAE for each forecast
BVAR6.err <- apply(BVAR.forc6$forecast, 1, "-", Ytmp1)
BVAR6.rmse <- sqrt(apply(array(BVAR6.err, c(6,4,ncol(BVAR6.err)))^2, 2, mean))
BVAR6.mae <- apply(abs(array(BVAR6.err, c(6,4,ncol(BVAR6.err)))), 2, mean)

BVARflat6.err <- apply(BVAR.flat.forc6$forecast, 1, "-", Ytmp1)
BVARflat6.rmse <- sqrt(apply(array(BVARflat6.err, c(6,4,ncol(BVARflat6.err)))^2, 2, mean))
BVARflat6.mae <- apply(abs(array(BVARflat6.err, c(6,4,ncol(BVARflat6.err)))), 2, mean)

univar6.err <- apply(univar.forc6.ensemble, 1, "-", Ytmp1)
univar6.rmse <- sqrt(apply(array(univar6.err, c(6,4,ncol(univar6.err)))^2, 2, mean))
univar6.mae <- apply(abs(array(univar6.err, c(6,4,ncol(univar6.err)))), 2, mean)

MSBVAR6.err <- apply(msbvar.forc6$forecast, 1, "-", Ytmp1)
MSBVAR6.rmse <-
    sqrt(apply(array(MSBVAR6.err,c(6,4,ncol(MSBVAR6.err)))^2, 2,
               mean))
MSBVAR6.mae <- apply(abs(array(MSBVAR6.err, c(6,4,ncol(MSBVAR6.err)))), 2, mean)

#############
BVAR12.err <- apply(BVAR.forc12$forecast, 1, "-", Ytmp2)
BVAR12.rmse <- sqrt(apply(array(BVAR12.err, c(12,4,ncol(BVAR12.err)))^2, 2, mean))
BVAR12.mae <- apply(abs(array(BVAR12.err, c(12,4,ncol(BVAR12.err)))), 2, mean)

BVARflat12.err <- apply(BVAR.flat.forc12$forecast, 1, "-", Ytmp2)
BVARflat12.rmse <- sqrt(apply(array(BVARflat12.err, c(12,4,ncol(BVARflat12.err)))^2, 2, mean))
BVARflat12.mae <- apply(abs(array(BVARflat12.err, c(12,4,ncol(BVARflat12.err)))), 2, mean)

univar12.err <- apply(univar.forc12.ensemble, 1, "-", Ytmp2)
univar12.rmse <- sqrt(apply(array(univar12.err, c(12,4,ncol(univar12.err)))^2, 2, mean))
univar12.mae <- apply(abs(array(univar12.err, c(12,4,ncol(univar12.err)))), 2, mean)

MSBVAR12.err <- apply(msbvar.forc12$forecast, 1, "-", Ytmp2)
MSBVAR12.rmse <-
    sqrt(apply(array(MSBVAR12.err,c(12,4,ncol(MSBVAR12.err)))^2, 2,
               mean))
MSBVAR12.mae <- apply(abs(array(MSBVAR12.err, c(12,4,ncol(MSBVAR12.err)))), 2, mean)

# CRPS
library(verification)
BVAR6.crps <- BVAR12.crps <- vector(ncol(Y.sample), mode="list")
BVARflat6.crps <- BVARflat12.crps <- vector(ncol(Y.sample),
                                            mode="list")
univar6.crps <- univar12.crps <- vector(ncol(Y.sample),
                                            mode="list")
MSBVAR6.crps <- MSBVAR12.crps <- vector(ncol(Y.sample), mode="list")

for(i in 1:ncol(Y))
{
    tmp <- crpsDecomposition(Ytmp1[,i], t(BVAR.forc6$forecast[,,i]))
    BVAR6.crps[[i]] <- crpsFromAlphaBeta(tmp$alpha, tmp$beta, tmp$heaviside0,
                                         tmp$heavisideN)

    tmp <- crpsDecomposition(Ytmp1[,i], t(BVAR.flat.forc6$forecast[,,i]))
    BVARflat6.crps[[i]] <- crpsFromAlphaBeta(tmp$alpha, tmp$beta, tmp$heaviside0,
                                             tmp$heavisideN)

    tmp <- crpsDecomposition(Ytmp1[,i], t(univar.forc6.ensemble[,,i]))
    univar6.crps[[i]] <- crpsFromAlphaBeta(tmp$alpha, tmp$beta, tmp$heaviside0,
                                           tmp$heavisideN)

    tmp <- crpsDecomposition(Ytmp1[,i], t(msbvar.forc6$forecast[,,i]))
    MSBVAR6.crps[[i]] <- crpsFromAlphaBeta(tmp$alpha, tmp$beta, tmp$heaviside0,
                                           tmp$heavisideN)
}

for(i in 1:ncol(Y))
{
    tmp <- crpsDecomposition(Ytmp2[,i], t(BVAR.forc12$forecast[,,i]))
    BVAR12.crps[[i]] <- crpsFromAlphaBeta(tmp$alpha, tmp$beta, tmp$heaviside0,
                                          tmp$heavisideN)

    tmp <- crpsDecomposition(Ytmp2[,i], t(BVAR.flat.forc12$forecast[,,i]))
    BVARflat12.crps[[i]] <- crpsFromAlphaBeta(tmp$alpha, tmp$beta, tmp$heaviside0,
                                              tmp$heavisideN)

    tmp <- crpsDecomposition(Ytmp2[,i], t(univar.forc12.ensemble[,,i]))
    univar12.crps[[i]] <- crpsFromAlphaBeta(tmp$alpha, tmp$beta, tmp$heaviside0,
                                            tmp$heavisideN)

    tmp <- crpsDecomposition(Ytmp2[,i], t(msbvar.forc12$forecast[,,i]))
    MSBVAR12.crps[[i]] <- crpsFromAlphaBeta(tmp$alpha, tmp$beta, tmp$heaviside0,
                                            tmp$heavisideN)

}



###################################################################
# Summaries of forecast comparisons
#
# Put these together for CRPS, RMSE, MAE
#
# Comparisons here are over
# a) variables (4)
# b) models (4)
# c) forecast horizons (2)
# d) performance metrics (RMSE, MAE, CRPS) (3)
#
###################################################################
tmp1 <- matrix(unlist(BVAR6.crps), nrow=3)[1,]    # BVAR6 CRPS
tmp2 <- matrix(unlist(BVAR12.crps), nrow=3)[1,]   # BVAR12 CRPS
tmp3 <- matrix(unlist(univar6.crps), nrow=3)[1,]  # Univariate 6 CRPS
tmp4 <- matrix(unlist(univar12.crps), nrow=3)[1,] # Univariate 12 CRPS
tmp5 <- matrix(unlist(BVARflat6.crps), nrow=3)[1,]
tmp6 <- matrix(unlist(BVARflat12.crps), nrow=3)[1,]
tmp7 <- matrix(unlist(MSBVAR6.crps), nrow=3)[1,]
tmp8 <- matrix(unlist(MSBVAR12.crps), nrow=3)[1,]


# Now make some tables
library(xtable)
rmse6 <- rbind(MSBVAR6.rmse, BVAR6.rmse, BVARflat6.rmse, univar6.rmse)
rmse12 <- rbind(MSBVAR12.rmse, BVAR12.rmse, BVARflat12.rmse, univar12.rmse)

mae6 <- rbind(MSBVAR6.mae, BVAR6.mae, BVARflat6.mae, univar6.mae)
mae12 <- rbind(MSBVAR12.mae, BVAR12.mae, BVARflat12.mae, univar12.mae)

crps6 <- rbind(tmp7, tmp1, tmp5, tmp3)
crps12 <- rbind(tmp8, tmp2, tmp6, tmp4)

colnames(rmse6) <- colnames(mae6) <- colnames(crps6) <- vnames
metrics6 <- cbind(rmse6, mae6, crps6)
rownames(metrics6) <- c("6MSBVAR", "6BVAR informed", "6BVAR flat", "6Univariate")

colnames(rmse12) <- colnames(mae12) <- colnames(crps12) <- vnames
metrics12 <- cbind(rmse12, mae12, crps12)
rownames(metrics12) <- c("12MSBVAR", "12BVAR informed", "12BVAR flat", "12Univariate")

print(xtable(rbind(metrics6, metrics12), digits=2),
      file="metrics.tex",
      )


###################################################################
# Compute the one-step ahead PITs -- so if nsteps=6,
# we are doing this for step 1 only to fit the assumptions of the
# metric!
###################################################################

# Compute the PITs for k=1 cases for each estimator and variable.
# This satisfies the conditional normality assumptions of the PIT
# statistic (loosely).

MSBVAR.pit <- univar.pit <- BVAR.pit <- BVARflat.pit <- matrix(0, gibbs, 4)

#### check serial correlation in the PITs!

# This does the PITs in a multivariate way
for(i in 1:gibbs)
{
    for(j in 1:4)
    {
        univar.pit[i,j] <- pmvnorm(lower=rep(-Inf,4), upper=Ytmp1[1,],
                                   mean=univar.forc6.ensemble[i,1,],
                                   sigma=var(univar.forc6.ensemble[,1,]))

        BVAR.pit[i,j] <- pmvnorm(lower=rep(-Inf,4), upper=Ytmp1[1,],
                                 mean=BVAR.forc6$forecast[i,1,],
                                 sigma=var(BVAR.forc6$forecast[,1,]))

        BVARflat.pit[i,j] <- pmvnorm(lower=rep(-Inf,4), upper=Ytmp1[1,],
                                      mean=BVAR.flat.forc6$forecast[i,1,],
                                      sigma=var(BVAR.flat.forc6$forecast[,1,]))

        MSBVAR.pit[i,j] <- pmvnorm(lower=rep(-Inf, 4),
                                   upper=Ytmp1[1,],
                                   mean=msbvar.forc6$forecasts[i,1,],
                                   sigma=var(msbvar.forc6$forecasts[,1,]))
    }
}

# These look awful
par(mfcol=c(4,4))
for(i in 1:4)
{
    hist(MSBVAR.pit[,i], main=vnames[i], breaks=seq(0,1, by=0.05),
         prob=TRUE, xlab="")
    hist(BVAR.pit[,i], main="", breaks=seq(0,1, by=0.05),
         prob=TRUE, xlab="")
    hist(BVARflat.pit[,i], main="", breaks=seq(0,1, by=0.05),
         prob=TRUE, xlab="")
    hist(univar.pit[,i], main="", breaks=seq(0,1, by=0.05),
         prob=TRUE, xlab="")
}


# Now do the univariate PIT versions over several periods
kk <- c(1,6,12)  # Sets the period of the forecast we want to consider
pdf(file="pits.pdf", width=6, height=6)
for(i in 1:length(kk))
{
    pd <- kk[i]
    par(mfcol=c(4,4), mar=c(2,5,2,1), omi=c(0, 0, 0.75, 0))
    for(j in 1:4)
    {
        sdtmp <- (msbvar.forc12$forecasts[,pd,j] -
                  mean(msbvar.forc12$forecast[,pd,j]))^2
        tmp <- pnorm(Ytmp2[pd,j], mean=msbvar.forc12$forecasts[,pd,j],
                     sd=sqrt(sdtmp))
        hist(tmp, main=vnames[j], breaks=seq(0,1,by=0.05), prob=TRUE,
             xlab="", ylab=ifelse(j==1, "MSBVAR", ""), ylim=c(0,13))

        sdtmp <- (BVAR.forc12$forecast[,pd,j] -
                  mean(BVAR.forc12$forecast[,pd,j]))^2
        tmp <- pnorm(Ytmp2[pd,j], mean=BVAR.forc12$forecast[,pd,j],
                     sd=sqrt(sdtmp))
        hist(tmp, main="", breaks=seq(0,1,by=0.05), prob=TRUE,
             xlab="", ylab=ifelse(j==1, "BVAR", ""), ylim=c(0,13))

        sdtmp <- (BVAR.flat.forc12$forecast[,pd,j] -
                  mean(BVAR.flat.forc12$forecast[,pd,j]))^2
        tmp <- pnorm(Ytmp2[pd,j], mean=BVAR.flat.forc12$forecast[,pd,j],
                     sd=sqrt(sdtmp))
        hist(tmp, main="", breaks=seq(0,1,by=0.05), prob=TRUE,
             xlab="", ylab=ifelse(j==1, "BVAR flat", ""), ylim=c(0,13))

        sdtmp <- (univar.forc12.ensemble[,pd,j] -
                  mean(univar.forc12.ensemble[,pd,j]))^2
        tmp <- pnorm(Ytmp2[pd,j], mean=univar.forc12.ensemble[,pd,j],
                     sd=sqrt(sdtmp))
        hist(tmp, main="", breaks=seq(0,1,by=0.05), prob=TRUE,
             xlab="", ylab=ifelse(j==1, "Univariate", ""), ylim=c(0,13))
    }
    mtext(paste("PIT for period ", pd), outer=TRUE, line=1)
}
dev.off()

###################################################################
# Compute the plot the VRHs
###################################################################
library(ensembleBMA)

pdf(file="straitsvrh.pdf", width=6, height=8)

# Make separate VRHs for each estimator.  We will do a separate plot
# for each forecast horizon.

par(mfcol=c(4,2), mar=c(4,2,2,2))

vrh.ms6 <- verifRankHist((array(aperm(msbvar.forc6$forecast,
                                      c(2,3,1)), c(48,5000))),
                         as.vector(Ytmp2))

vrh.bvar6 <- verifRankHist((array(aperm(BVAR.forc6$forecast,
                                        c(2,3,1)), c(48,5000))),
                           as.vector(Ytmp2))

vrh.bvarf6 <- verifRankHist((array(aperm(BVAR.flat.forc6$forecast,
                                         c(2,3,1)), c(48,5000))),
                            as.vector(Ytmp2))

vrh.uni6 <- verifRankHist((array(aperm(univar.forc6.ensemble,
                                       c(2,3,1)), c(48,5000))),
                          as.vector(Ytmp2))

vrh.ms12 <- verifRankHist((array(aperm(msbvar.forc12$forecast,
                                       c(2,3,1)), c(48,5000))),
                          as.vector(Ytmp2))

vrh.bvar12 <- verifRankHist((array(aperm(BVAR.forc12$forecast,
                                         c(2,3,1)), c(48,5000))),
                            as.vector(Ytmp2))

vrh.bvarf12 <- verifRankHist((array(aperm(BVAR.flat.forc12$forecast,
                                          c(2,3,1)), c(48,5000))),
                             as.vector(Ytmp2))

vrh.uni12 <- verifRankHist((array(aperm(univar.forc12.ensemble,
                                        c(2,3,1)), c(48,5000))),
                       as.vector(Ytmp2))

dev.off()

# Compute and plot the sharpness diagrams
# Do this with violin plots.

library(wvioplot)

# Generate violin plots for the estimators at period 1 for each
# variable -- 12 step forecast
estnames <- c("MSBVAR", "BVAR", "BVAR\nflat", "Univariate")

pdf(file="sharpnessviolins.pdf", width=6.45, height=8)

par(mfcol=c(4,3), cex.axis=0.65, mar=c(2,4,2,2))
for(i in 1:4)
{
    wvioplot(msbvar.forc12$forecasts[,1,i],
             BVAR.forc12$forecast[,1,i],
             BVAR.flat.forc12$forecast[,1,i],
             univar.forc12.ensemble[,1,i],
             colMed="black",
             at=seq(1, 8, by=2),
             wex=1.5,
             names=estnames, clip=FALSE, adjust=1)
    abline(h=Ytmp2[1,i])
    mtext(vnames[i], side=2, line=2, outer=FALSE)
    if(i==1) mtext("Period 1", line=0.75)
}

# Same thing for period 6
#par(mfrow=c(2,2), cex.axis=0.8, mar=c(2,2,2,2))
for(i in 1:4)
{
    wvioplot(msbvar.forc12$forecasts[,6,i],
             BVAR.forc12$forecast[,6,i],
             BVAR.flat.forc12$forecast[,6,i],
             univar.forc12.ensemble[,6,i],
             colMed="black",
             at=seq(1, 8, by=2),
             wex=1.5,
            names=estnames, clip=FALSE, adjust=1)
    abline(h=Ytmp2[6,i])
    if(i==1) mtext("Period 6", line=0.75)
}

# Period 12
#par(mfrow=c(2,2), cex.axis=0.8, mar=c(2,2,2,2))
for(i in 1:4)
{
    wvioplot(msbvar.forc12$forecasts[,12,i],
             BVAR.forc12$forecast[,12,i],
             BVAR.flat.forc12$forecast[,12,i],
             univar.forc12.ensemble[,12,i],
             colMed="black",
             at=seq(1, 8, by=2),
             wex=1.5,
            names=estnames, clip=FALSE, adjust=1)
    abline(h=Ytmp2[12,i])
    if(i==1) mtext("Period 12", line=0.75)
}

dev.off()

# Compute widths of these intervals as well for a table.
# Use 5, 16, 25, 50, 75, 84, 95 intervals.
# Note reversal of quantiles!

# Period 1
msbvarlo1 <- apply(msbvar.forc12$forecasts[,1,], 2, quantile,
                   probs=c(0.05, 0.16, 0.25))
msbvarhi1 <- apply(msbvar.forc12$forecasts[,1,], 2, quantile,
                   probs=c(0.95, 0.84, 0.75))

msbvarwd1 <- msbvarhi1-msbvarlo1

BVARlo1 <- apply(BVAR.forc12$forecast[,1,], 2, quantile,
                   probs=c(0.05, 0.16, 0.25))
BVARhi1 <- apply(BVAR.forc12$forecast[,1,], 2, quantile,
                   probs=c(0.95, 0.84, 0.75))

BVARwd1 <- BVARhi1-BVARlo1

BVARflatlo1 <- apply(BVAR.flat.forc12$forecast[,1,], 2, quantile,
                   probs=c(0.05, 0.16, 0.25))
BVARflathi1 <- apply(BVAR.flat.forc12$forecast[,1,], 2, quantile,
                   probs=c(0.95, 0.84, 0.75))

BVARflatwd1 <- BVARflathi1-BVARflatlo1

univarlo1 <- apply(univar.forc12.ensemble[,1,], 2, quantile,
                   probs=c(0.05, 0.16, 0.25))
univarhi1 <- apply(univar.forc12.ensemble[,1,], 2, quantile,
                   probs=c(0.95, 0.84, 0.75))

univarwd1 <- univarhi1-univarlo1

# Period 6
msbvarlo6 <- apply(msbvar.forc12$forecasts[,6,], 2, quantile,
                   probs=c(0.05, 0.16, 0.25))
msbvarhi6 <- apply(msbvar.forc12$forecasts[,6,], 2, quantile,
                   probs=c(0.95, 0.84, 0.75))

msbvarwd6 <- msbvarhi6-msbvarlo6

BVARlo6 <- apply(BVAR.forc12$forecast[,6,], 2, quantile,
                   probs=c(0.05, 0.16, 0.25))
BVARhi6 <- apply(BVAR.forc12$forecast[,6,], 2, quantile,
                   probs=c(0.95, 0.84, 0.75))

BVARwd6 <- BVARhi6-BVARlo6

BVARflatlo6 <- apply(BVAR.flat.forc12$forecast[,6,], 2, quantile,
                   probs=c(0.05, 0.16, 0.25))
BVARflathi6 <- apply(BVAR.flat.forc12$forecast[,6,], 2, quantile,
                   probs=c(0.95, 0.84, 0.75))

BVARflatwd6 <- BVARflathi6-BVARflatlo6

univarlo6 <- apply(univar.forc12.ensemble[,6,], 2, quantile,
                   probs=c(0.05, 0.16, 0.25))
univarhi6 <- apply(univar.forc12.ensemble[,6,], 2, quantile,
                   probs=c(0.95, 0.84, 0.75))

univarwd6 <- univarhi6-univarlo6

# Period 12
msbvarlo12 <- apply(msbvar.forc12$forecasts[,12,], 2, quantile,
                   probs=c(0.05, 0.16, 0.25))
msbvarhi12 <- apply(msbvar.forc12$forecasts[,12,], 2, quantile,
                   probs=c(0.95, 0.84, 0.75))

msbvarwd12 <- msbvarhi12-msbvarlo12

BVARlo12 <- apply(BVAR.forc12$forecast[,12,], 2, quantile,
                   probs=c(0.05, 0.16, 0.25))
BVARhi12 <- apply(BVAR.forc12$forecast[,12,], 2, quantile,
                   probs=c(0.95, 0.84, 0.75))

BVARwd12 <- BVARhi12-BVARlo12

BVARflatlo12 <- apply(BVAR.flat.forc12$forecast[,12,], 2, quantile,
                   probs=c(0.05, 0.16, 0.25))
BVARflathi12 <- apply(BVAR.flat.forc12$forecast[,12,], 2, quantile,
                   probs=c(0.95, 0.84, 0.75))

BVARflatwd12 <- BVARflathi12-BVARflatlo12

univarlo12 <- apply(univar.forc12.ensemble[,12,], 2, quantile,
                   probs=c(0.05, 0.16, 0.25))
univarhi12 <- apply(univar.forc12.ensemble[,12,], 2, quantile,
                   probs=c(0.95, 0.84, 0.75))

univarwd12 <- univarhi12-univarlo12


# Now make a table of the CI widths

# Get 50% interval, then 68%... etc.
wd1 <- rbind(msbvarwd1[3,], BVARwd1[3,], BVARflatwd1[3,], univarwd1[3,],
             msbvarwd1[2,], BVARwd1[2,], BVARflatwd1[2,], univarwd1[2,],
             msbvarwd1[1,], BVARwd1[1,], BVARflatwd1[1,],
             univarwd1[1,])

wd6 <- rbind(msbvarwd6[3,], BVARwd6[3,], BVARflatwd6[3,], univarwd6[3,],
             msbvarwd6[2,], BVARwd6[2,], BVARflatwd6[2,], univarwd6[2,],
             msbvarwd6[1,], BVARwd6[1,], BVARflatwd6[1,],
             univarwd6[1,])

wd12 <- rbind(msbvarwd12[3,], BVARwd12[3,], BVARflatwd12[3,], univarwd12[3,],
             msbvarwd12[2,], BVARwd12[2,], BVARflatwd12[2,], univarwd12[2,],
             msbvarwd12[1,], BVARwd12[1,], BVARflatwd12[1,],
             univarwd12[1,])

wd <- cbind(wd1, wd6, wd12)

colnames(wd) <- rep(vnames, 3)

print(xtable(wd, align="l|rrrr|rrrr|rrrr", digits=2),
      file="CIwidths.tex")


# Save
save.image("China-Taiwan-19982011-Forecast.RData")
